//  StartESFrame.java - jes Open Foundation version
//  Pietro Terna (start June 2004; this version September 2006)
//  Dipartimento di Scienze economiche e finanziarie G.Prato
//  Universita' di Torino
//  Licensed under the Academic Free License version 1.2
//  (see license.txt in the distribution and 
//  www.opensource.org/licenses/academic.php)

//  Some Swarm building ideas and comments come from jHeatbugs (Java Heatbugs
//  application. Copyright 1999-2000 Swarm Development Group.)

import swarm.Globals;
import java.io.*;
import swarm.defobj.LispArchiverImpl; //to assure compatibility with Fedora 5 rpm

/**
 * The StartESFrame class contains main().
 * We follow here the typical Swarm structure with main() (in Start...
 * as a convention) generating the Observer and the Observer
 * generating the Model.<br>
 *
 * @author Pietro Terna
 */
public class StartESFrame
    {
	/** the observer in our application */
	public static ESFrameObserverSwarm eSFrameObserverSwarm;

	/** being verbose? are whole zero time orders invisibiles? */
	public static boolean verbose, wholeZeroTimeOrdersAreInvisible, writeNumberOfUnitsToFile;

	/**
	 * The main() function is the top-level place where everything starts.
	 */
	public static void main (String[] args) {


	    // Swarm initialization
	    Globals.env.initSwarm ("jesopenfoundation", "v.0.1.72", 
				   "pietro.terna@unito.it",args);

      Globals.env.lispAppArchiver = new LispArchiverImpl(Globals.env.globalZone, 
       	   "jesopenfoundation.scm"); //to assure compatibility with Fedora 5 rpm


	    // creating the ESFrameObserverSwarm via lispAppArchiver, which
	    // refers to the jesopenfoundation.scm file where we can modify parameters
	    // in a stable way without recompile the interested class
	    eSFrameObserverSwarm =
		(ESFrameObserverSwarm) Globals.env.lispAppArchiver.getWithZone$key(
				       Globals.env.globalZone, "eSFrameObserverSwarm");
	    // instead of using the direct constructor way
	    // eSFrameObserverSwarm = new
	    //                      eSFrameObserverSwarm (Globals.env.globalZone);

	    // to save control panel position
	    Globals.env.setWindowGeometryRecordName
		(eSFrameObserverSwarm, "eSFrameObserverSwarm");
	    
	    // build objets into the obsever
	    eSFrameObserverSwarm.buildObjects ();
	    // verbose now is true/false, following the choice made
	    // in the Observer; the same botj for ls ESwholeZeroTimeOrdersAreInvisible
	    // and writeNumberOfUnitsToFile
	    
	    // build actions into the observer
	    eSFrameObserverSwarm.buildActions ();

	    // activate
	    eSFrameObserverSwarm.activateIn (null);

	    // go() in eSFrameObserver is a method inherited from class
	    // swarm.simtoolsgui.GUISwarmImpl
	    // go() start the activity running, and also handle user requests
	    // via the control panel.
	    eSFrameObserverSwarm.go();
	
	    // concluding the activity we drop the observer and then the model
	    // dropping the observer we use a redefined drop method, which
	    // starts with the 'getControlPanel ().setStateStopped ();' instruction
	    // necessary to avoid a Tcl error, appearing in W2000 and XP
	    // we have so to press Quit twice
	    eSFrameObserverSwarm.drop();
	}
}
