//  ComputationalAssembler.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;


/**
 * The ComputationalAssembler class instances make 
 * computational processes; we have a computationa assembler instance for
 * each unit
 *
 * @author Pietro Terna 
 */
public class ComputationalAssembler extends ComputationalAssemblerBasic{

    /**
     * the constructor for ComputationalAssembler
     */

    public ComputationalAssembler (Zone aZone, OrderDistiller od, ESFrameModelSwarm mo)
    {
	// Call the constructor for the parent class.
	super(aZone, od, mo);

    }

    /** drop the computational assembler */
    public void drop(){

	InformationRepository.addToHibernatedComputationalAssemblerList(this);
    }

    //
    // this version of ComputationalAssembler.java is for tutorial-step3a
    //


    /** eating 
     *  <br><br>
     *  computational operations with code -1202 (a code for the prey-predator tutorial)
     *  <br><br>
     *
     *  this computational code acts via -1102 code to drop the unit the order is in, 
     *  with the probability set in position 0,0 of the first received matrix and 
     *  increases the  position 0,1 of the second received matrix by 1, to count the 
     *  dropped units; finally, it adds 1 to pos. 0,0 of the sending unit 
     *  memory matrix, to count its eating acts; done is set to true in c1102
     */
    public void c1202()
    {
	Order theCurrentOrder;
	Unit theUnitTheOrderComesFrom;
	int layer=0;
	float count;

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=2)
	    {
		System.out.println("Code -1202 requires 2 matrixes; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	c1102();

	theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
	theUnitTheOrderComesFrom=theCurrentOrder.getTheUnitTheOrderComesFrom();
	count=(theUnitTheOrderComesFrom.getMemoryMatrix()).getValue(layer,0,0);
	count++;
	(theUnitTheOrderComesFrom.getMemoryMatrix()).setValue(layer,0,0,count);

    } // end c1202


    /** perishing (unsufficient food, i.e. energy)
     *  <br><br>
     *  computational operations with code -1204 (a code for the prey-predator tutorial)
     *  <br><br>
     *
     *  this computational code acts via c1102 code to drop the unit the order is in, if
     *  pos. 0,0 value of the unit  memory matrix is less then the parameter in 1,2 
     *  (minEnergy) of the second received matrix; c1102 acts with the probability set 
     *  in position 1,0 of the first received matrix (internal trick: c1102 looks for 
     *  0+rd,0+cd using temporary rd (the row displacement) set to 1; c1102 increases 
     *  the  position 1,1 (always via the displacement, being the internal coordinates 
     *  0+rd,1+cd, of the second received matrix by 1, to count the dropped units; 
     *  done is set to true in c1102 or here in c1204
     */
    public void c1204()
    {
	Order theCurrentOrder;
	Unit theUnitTheOrderIsIn;
	int layer=0; float minEnergy;

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=2)
	    {
		System.out.println("Code -1204 requires 2 matrixes; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	mm0=(MemoryMatrix) pendingComputationalSpecificationSet.
	    getMemoryMatrixAddress(0);

	minEnergy=mm0.getValue(layer,1,2);

	theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
	theUnitTheOrderIsIn=theCurrentOrder.getTheUnitTheOrderIsIn();

	if (minEnergy > (theUnitTheOrderIsIn.getMemoryMatrix()).getValue(layer,0,0))
	    {
		rd=1; // prob is in 1,0, but c1102() looks for it at 0+rd,0+cd (with cd set to 0)
		      // also the count of the dropped units is in 0+rd,1+cd (with cd set to 0)
		c1102();
		rd=0;
	    }
	else done=true; // doing nothing

    } // end c1204

    /** consuming (decreasing energy)
     *  <br><br>
     *  computational operations with code -1205 (a code for the prey-predator tutorial)
     *  <br><br>
     *
     *  this computational code acts with the probability set in position 2,0 of the unique
     *  received matrix to decrease of 1 unit the position 0,0 of the memory matrix of the unit
     *  the order is in; 
     *  finally done is set to true
     */
    public void c1205()
    {
	Order theCurrentOrder;
	Unit theUnitTheOrderIsIn;
	int layer=0; float prob, energy;

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=1)
	    {
		System.out.println("Code -1205 requires 1 matrix; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	mm0=(MemoryMatrix) pendingComputationalSpecificationSet.
	    getMemoryMatrixAddress(0);
	prob = mm0.getValue(layer,2,0);

	if(prob >= Globals.env.uniformDblRand.getDoubleWithMin$withMax(0, 1))
	    {
		theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
		theUnitTheOrderIsIn=theCurrentOrder.getTheUnitTheOrderIsIn();
		// getting unit energy and decreasing it
		energy=(theUnitTheOrderIsIn.getMemoryMatrix()).getValue(layer,0,0);
		(theUnitTheOrderIsIn.getMemoryMatrix()).setValue(layer,0,0,energy-1);
	    }

	done=true;

    } // end c1205


    /** reproducing (creating a copy of themselves near to their position, if their 
	energy is sufficient)
     *  <br><br>
     *  computational operations with code -1206 (a code for the prey-predator tutorial)
     *  <br><br>
     *
     *  this computational code acts via the c1103 code to create, with the probability set in 
     *  position 3,0 of the first received matrix (c1103 see it with rd=3 as 0+rd,0+cd), a copy
     *  of the unit the order is in, near to it, if comparing position 0,0 of the unit memory 
     *  matrix and position 3,2 of the second received matrix, i.e. if energy >= requested level 
     *  to reproduce; the count of the created unit is reported in position 3,1 of the second
     *  received matrix  (via c1103 with rd=3)
     *
     *  done is set to true in c1103 or here in c1206
     */
    public void c1206()
    {
	Order theCurrentOrder;
	Unit theUnitTheOrderIsIn;
	int layer=0; float energyReproducingLevel, energy;

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=2)
	    {
		System.out.println("Code -1206 requires 2 matrixes; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	mm0=(MemoryMatrix) pendingComputationalSpecificationSet.
	    getMemoryMatrixAddress(1);
	energyReproducingLevel = mm0.getValue(layer,3,2);

	theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
	theUnitTheOrderIsIn=theCurrentOrder.getTheUnitTheOrderIsIn();
	// getting unit energy and decreasing it
	energy=(theUnitTheOrderIsIn.getMemoryMatrix()).getValue(layer,0,0);
	
	// reproducing if energy is > the energyReproducingLevel
	if (energy>=energyReproducingLevel)
	    {
		rd=3;
		c1103();
		rd=0;
	    }
	else done=true;

    } // end c1206

    /** this code simply call the highly usable c1199 computational code, using rd=1 to direct
     *  the called step to the correct row of the input matrixes 
     */
    public void c1299(){

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=2)
	    {
		System.out.println("Code -1299 requires 2 matrixes; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	rd=1;
	c1199();
	rd=0;

    } // end c1299

}
