// Order.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
import swarm.Selector;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;

/**
 * The form containing our Order metaphoric object
 *
 * @author Pietro Terna (with Paolo Pelligra and Dario Landini contibutions)
 */
public class Order  extends SwarmObjectImpl implements Cloneable{

    /** the number of steps in the product recipe */
    public int stepNumber;
    /** the 'name' of the recipe (may be used by OrderDistiller, as a way
     *  to identify recipes */
    public String recipeName;
    /** the layer to which the order belongs; this value is not used here
     *  but in jesframe; here is kept (always 0) for compatibility reasons */
    public int orderLayer; 
    /** the number of this order */
    public int orderNumber;
   /** the day in which the order as been generated */
    public int startingTimeUnit;
    /** integer vector, containing the recipe steps */
    public int[] orderRecipe;
    /** a temporary vector to redefine the dimension of orderRecipe
     *  if necessary (using  "p" or "c" or "||" or "&&" steps) */ 
    public int[] orderRecipeTmp;
    /** a temporary vector to redefine the dimension of orderState
     *  if necessary (using  "p" or "c" or "||" or "&&" steps) */ 
    public int[] orderStateTmp;
    /** integer vector, recording the performed phases */
    public int[] orderState;
    /** number of ticks the order is in this unit as unsent */
    public int numberOfTicksAsUnsent;

    /** the unit the order come from (null at the beginning, coming from distiller or
     *  generator) NB must be a different unit, i.e. we do not register here the assignnment
     *  to the same unit or to a unit executing the same step */
    public Unit theUnitTheOrderComesFrom;
    /** the unit the order is in, set from assign() in AssigningTool */
    public Unit theUnitTheOrderIsIn;

    /** computational specification set */
    public ComputationalSpecificationSet aComputationalSpecificationSet;
    /** the list of the computational specification set */
    public ListImpl computationalSpecificationSetList;
    /** the vector of memory matrix addresses to be set in a
     *  computational specification set */
    public Object [] matrixAddress;
    /** the dimension of matrixAddress*/
    public int dim;

    // iterators
    ListIndex computationalSpecificationSetListIndex;

    /** the model */
    ESFrameModelSwarm eSFrameModelSwarm;

    /** indexes used in a common way in the arrangeForComputation method */
    public int i, j, jj, k;

    /** incoming recipe structure; we make a local copy of it, from r0 which
     *  is in the constructor paramenter list, to r */
    public int[] r;

    /**
     * the constructor for Order
     */
    public Order (Zone aZone, int n, int sd, int sn, int [] r0, 
		  ESFrameModelSwarm model) {
	// Call the constructor for the Order parent class
	super(aZone);
	
	init(aZone, n, sd, sn, r0, model);
    }

    /** managing the constructor content */
    public void init (Zone aZone, int n, int sd, int sn, int [] r0, 
		      ESFrameModelSwarm model)
    {
      recipeName="noName";
      // the layer to which the order belongs; this value is not used here
      //  but in jesframe; here is kept (always 0) for compatibility reasons
      orderLayer=0; 
      theUnitTheOrderComesFrom=null;
      theUnitTheOrderIsIn=null;
      
      r = new int[r0.length];
      for (i=0;i<r0.length;i++)
	  {
	      r[i]=r0[i];
	  }
      orderNumber=n;
      stepNumber=sn;
      startingTimeUnit=sd;
      orderRecipe= new int[stepNumber];
      orderState = new int[stepNumber];
      // the model
      eSFrameModelSwarm = model;
      
      computationalSpecificationSetList = new ListImpl(getZone());
      computationalSpecificationSetListIndex = 
	  computationalSpecificationSetList.listBegin(getZone());
      
      dim=eSFrameModelSwarm.getTotalMemoryMatrixNumber();
      if(dim <10) dim=10;
      matrixAddress = new Object[dim];

      i=0; j=0;

     // copying the recipe
     while (j<stepNumber)
	 {
	     if (r[j]==0){
		 System.out.
		     println("Internal error in order generation: " + 
			     "look at the constructor of Order class.");
		 MyExit.exit(1);
	     }

	     if (r[j]>0) orderRecipe[i]=r[j];

	     // "c" case
	     if (r[j]<0)
		 {
		     if(r[j]>=-1999 && r[j]<=-1001)
			 arrangeForComputation();               // "c" case
		     else
			 {
			     System.out.println("Invalid negative code in a recipe");
			     MyExit.exit(1);
			 }

		 }
	     i++; j++;
	 }

     // i is now the internal length of vectors, so we redefine the 
     // stepNumber
     stepNumber=i;

     // redefine orderRecipe vector
     orderRecipeTmp = new int[stepNumber];
     System.arraycopy(orderRecipe,0,orderRecipeTmp,0,stepNumber);
     orderRecipe=orderRecipeTmp;

     // redefine orderState vector
     orderStateTmp = new int[stepNumber];
     System.arraycopy(orderState,0,orderStateTmp,0,stepNumber);
     orderState=orderStateTmp;

     // announcing the order presence to the console
     if(StartESFrame.verbose)
	 System.out.println("New order (#" + orderNumber + ")");

     // setting the state vector (0=empty; not zero=done)
     if(StartESFrame.verbose)System.out.print("orderState  vector ");
     for (i=1;i<=stepNumber;i++) {
	 orderState[i-1]=0;
	 if(StartESFrame.verbose)System.out.print(orderState[i-1] + " ");
     }
     if(StartESFrame.verbose)System.out.println();

     // announcing the recipe vector
     if(StartESFrame.verbose)
	 {
	 System.out.print("orderRecipe vector ");
	 for (i=1;i<=stepNumber;i++)
	     System.out.print(orderRecipe[i-1] + " ");
	 System.out.println();
     }
  }

    /** the internal method dealing with computation */
    public void arrangeForComputation()	
    {

	jj=j;

	// r[jj]   contains the computational code
	// r[jj+1] contains the number of matrixes used
	j += r[j+1] + 2;
	if (! (j<stepNumber)){System.out.
	println(
	"Internal dimension error analysing the order: " + 
	"look at arrangeForComputation() method of Order class.");
	MyExit.exit(1);
	}

	// making a ComputationalSpecificationSet object, linked
	// at the position 'i' of the recipe
	if(InformationRepository.getCountFromHibernatedComputationalSpecificationSetList()==0)	
	    aComputationalSpecificationSet = new 
		ComputationalSpecificationSet(getZone(), this, i);
	else 
	    {
		aComputationalSpecificationSet=InformationRepository.
		    removeFromHibernatedComputationalSpecificationSetList();
		aComputationalSpecificationSet.init(getZone(), this, i);
	    }

	
	computationalSpecificationSetList.
	    addLast(aComputationalSpecificationSet);

	// building the vector of memory matrix addresses
	for (int m=0; m<r[jj+1];m++)
	    {
		if(m>=dim)
		    {
			System.out.println("Too much matrixes in a computational step list: "+
					   "max 10 or the matrix number, if > 10.");
			MyExit.exit(2);
		    }
		matrixAddress[m]=eSFrameModelSwarm.getMemoryMatrix(r[jj+2+m]);
	    }
	
	// setting specifications into 
	// aComputatatioanlSpecificazioneSet
	aComputationalSpecificationSet.
	    setSpecificationSet(r[jj+1], matrixAddress, r[jj]);
	
	// being after a "c" specification, stop execution
	// if the unit is an endUnit
	/* this does not work here
	  for(k=0;k<endUnitList.getCount();k++)
	    if(r[j]==((EndUnit) endUnitList.atOffset(k)).
	       getUnitNumber()){
		System.out.println(
			   "Syntax error in Order # " + getOrderNumber());
		System.out.println("An endUnit cannot follow " + 
			   "immediately after a computational step");
		MyExit.exit(1);
	    }
	*/

	// copying the recipe
	orderRecipe[i] = r[j];
	
    }

  /**
   * production next step in Order (it is in the orderRecipe vector,
   * at the same index value in which we find the first 0 in orderState
   * vector)
   */
    public int getNextStep () {

	int i, nextStep;

	nextStep=0;
	for (i=0;i< stepNumber && nextStep==0;i++)
	    if (orderState[i]==0)nextStep=getOrderRecipe(i);

	// if no 0 position can be found the order is finished
	// in this case we return 0

	if(nextStep>=1000000000)nextStep-=1000000000;
	return nextStep;
  }

  /**
   * production done step in Order (it is in the orderRecipe vector,
   * at the index value in which we find the first 0 in orderState
   * vector minus 1; if no step has been done, the method returns -999999)
   */
    public int getDoneStep () {

	int i, doneStep, firstZeroPosition;

	firstZeroPosition=-1;
	for (i=0;i< stepNumber && firstZeroPosition==-1;i++)
	    if (orderState[i]==0)firstZeroPosition=i;

	// if no 0 position can be found, the order is finished;
	// in this case we return the last possible step
	if(firstZeroPosition==-1)doneStep=getOrderRecipe(stepNumber-1);

	// if the first 0 is in 0 position, no done step exists, so we return
	// -999999
	else 
	    if(firstZeroPosition==0)doneStep=-999999;
	    else 
		if(orderState[firstZeroPosition-1]==-1)
		    doneStep=-1;  // we are on the border of an 
	                          // unused 'or' branch
		else doneStep=getOrderRecipe(firstZeroPosition-1);

	return doneStep;
  }

    /**
     *  getting the layer to which the order belongs
     */
    public int getOrderLayer(){
	return orderLayer; // this kind of layers (order layers) is use in jesframe
	                   // here is kept only for compatibility reasons; this
                           // method returns always 0
    }

    /**
     * Done step in Order
     */
    public void setDoneStep (int u) {

	int i, done;
	if(StartESFrame.verbose)
	    System.out.println("done step in order #"+getOrderNumber());
	done=0;
	for (i=0;i<= stepNumber && done==0;i++)
	    if (orderState[i]==0)
		{orderState[i]=u;
		done=1;
		}
    }

  /**
   * returning the orderNumber of this order
   */
    public int getOrderNumber () {
	return orderNumber;
  }

  /**
   * returning the stepNumber
   */
    public int getStepNumber () {

	return stepNumber;
  }

  /**
   * returning the startingTimeUnit
   */
    public int getStartingTimeUnit() {

	return startingTimeUnit;
  }


   /** evaluate the number of direct steps done */
    public float getNumberOfDirectStepsDone(){
	float globalOrderState, d, dRatio; 
	int i;
 
	globalOrderState=0; d=0; dRatio=0;
	for(i=0; i<orderState.length; i++){ /* not accounting for zero time steps*/
	    if(getOrderState(i)>0 && getOrderRecipe(i)<1000000000)
		globalOrderState++;
	}


	if(StartESFrame.verbose)
	    System.out.println(
			       "in Order # " + getOrderNumber());

	return globalOrderState;
   }


    /** getting the content of orderRecipe[i] */
    public int getOrderRecipe(int i){
	if(i>=stepNumber){
	    System.out.println(
	 "Error in getting a recipe step in an order (max length exceeded)");
	    MyExit.exit(1);
	}
	return orderRecipe[i];
    }

    /** getting the content of orderState[i] */
    public int getOrderState(int i){
	if(i>=stepNumber){
	    System.out.println(
	 "Error in getting a state value in an order (max length exceeded)");
	    MyExit.exit(1);
	}
	return orderState[i];
    }


    /** settinging the name of the recipe */
    public void setRecipeName(String s){
	recipeName=s;
    }

    /** retrieving the name of the recipe */
    public String getRecipeName(){
	return recipeName;
    }

    /**
     * check if next step in Order uses a computational specification set
     */
    public boolean checkIfNextStepUsesAComputationalSpecificationSet() {
	
	if(getPendingComputationalSpecificationSet() != null)
	    return true;
	else
	    return false;
  }

    /** 
     *  check if next step in order is a zero time step
     */
    public boolean checkZeroTimeProduction() {

	int i, nextStep;

	nextStep=0;
	for (i=0;i< stepNumber && nextStep==0;i++)
	    if (orderState[i]==0)nextStep=getOrderRecipe(i);

	// if no 0 position can be found the order is finished
	// in this case nextStep = 0

	if(nextStep>=1000000000)return true;
	else return false;
    }


    /** getting the computational specification set related to the nextStep
     *  (the current one, if the order is in a waiting list for next step
     *  to be accomplished)<br>
     *  if no computational specification set exists for the next step,
     *  a null value is returned */
    public Object getPendingComputationalSpecificationSet()
    {
	int i, position;
	ComputationalSpecificationSet pendingComputationalSpecificationSet;

	position=-1;
	for (i=0;i< stepNumber && position==-1;i++)
	    if (orderState[i]==0)position=i;

	pendingComputationalSpecificationSet = null;

	if(computationalSpecificationSetList.getCount()>0){
	    computationalSpecificationSetListIndex.setOffset(0);
	    for(i=0;i<computationalSpecificationSetList.getCount();i++){
		if(position == ((ComputationalSpecificationSet)
				computationalSpecificationSetListIndex.get()).
		   getPositionInRecipe())
		    pendingComputationalSpecificationSet =
			(ComputationalSpecificationSet)
			computationalSpecificationSetList.atOffset(i);
		computationalSpecificationSetListIndex.next();
	    }
	}
	return pendingComputationalSpecificationSet;
    }

    /** set the unit the order is in (from assign() in AssigningTool) */
    public void setTheUnitTheOrderIsIn(Unit in)
    {
	if(theUnitTheOrderIsIn != in)
	    theUnitTheOrderComesFrom = theUnitTheOrderIsIn;
	theUnitTheOrderIsIn=in;

	if(StartESFrame.verbose)
	    {
		if(theUnitTheOrderComesFrom == null)
		    System.out.println("Order "+getOrderNumber()+
				       " is coming from distiller/generator and is in unit number "
				       + theUnitTheOrderIsIn.getUnitNumber());
		else
		    System.out.println("Order "+getOrderNumber()+
				       " is coming from unit number " + 
				       theUnitTheOrderComesFrom.getUnitNumber() + 
				       " and is in unit number " + theUnitTheOrderIsIn.getUnitNumber());
	    }
    }

    /** get the unit the order is in */
    public Unit getTheUnitTheOrderIsIn()
    {
	return theUnitTheOrderIsIn;
    }

    /** get the unit the order comes from */
    public Unit getTheUnitTheOrderComesFrom()
    {
	return theUnitTheOrderComesFrom;
    }

    /** setting (initializing) the number of ticks that the order is
     *  in the dailyProductionList of a Unit; the setting is made in unitStep2; the
     *  initial value is set in unitStep1 */
    public void setNumberOfTicksAsUnsent(int n){
	numberOfTicksAsUnsent=n;
    }

    /** getting the number of ticks that the order is 
     *  in the dailyProductionList of a Unit, as unsent */
    public int getNumberOfTicksAsUnsent(){
	return numberOfTicksAsUnsent;
    }

    /** drop the order */
    public void drop(){

	// dropping our computationalSpecificationSet
	while(computationalSpecificationSetList.getCount()>0)
	    ((ComputationalSpecificationSet) 
	     computationalSpecificationSetList.removeFirst()).drop();

	//	super.drop();
		InformationRepository.addToHibernatedOrderList(this);
    }

}
