// UnitActivitySpace.java
// Defines the class as a subclass of Discrete2dImpl.

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.space.Discrete2dImpl;

public class UnitActivitySpace extends Discrete2dImpl
{
    public int [][] rotationMatrix, aSpace, bSpace;
    public int [] directionVector, directionVector0;
    int xSize, ySize;


    // The constructor for the visibility space.  Create a space in zone
    // aZone with dimensions xSize and ySize.  Note that a
    // food space is based on the Swarm's Discrete2dImpl class.
    public UnitActivitySpace (Zone aZone, int xS, int yS)
    {
	// Call the constructor for the parent class and then fill the
	// lattice with zeros.
	super(aZone, xS, yS);
	fastFillWithValue(0);

	xSize=xS; ySize=yS;
	rotationMatrix = new int [2][2];
	rotationMatrix [0][0]=0;
	rotationMatrix [0][1]=1;
	rotationMatrix [1][0]=-1;
	rotationMatrix [1][1]=0;

	directionVector = new int [2];
	directionVector0 = new int [2];
	aSpace = new int [xSize][ySize];
	bSpace = new int [xSize][ySize];
    }

    public void fillWithZero (){
	fastFillWithValue(0);
    }
	
    // to build histograms (left)
    public void showLeftHistogram(int x, int y, int v)
    {
	int i, xx;
	if(InformationRepository.getNoHistograms())return;
	if (v == 0) return;

	for (i=1;i<=v;i++)
	    {
		xx=CoordinateManager.check(getSizeX(), x-i);
		putValue$atX$Y(3, xx, y);
	    }
    }

    // to build histograms (right)
    public void showRightHistogram(int x, int y, int v)
    {
	int i, xx;

	if(InformationRepository.getNoHistograms())return;
	if (v == 0) return;

	for (i=1;i<=v;i++)
	    {
		xx=CoordinateManager.check(getSizeX(), x+i);
		putValue$atX$Y(4, xx, y);
	    }
    }

    // to build histograms (down)
    public void showDownHistogram(int x, int y, int v)
    {
	int i, yy;

	if(InformationRepository.getNoHistograms())return;
	if (v == 0) return;

	for (i=1;i<=v;i++)
	    {
		yy=CoordinateManager.check(getSizeY(), y+i);
		putValue$atX$Y(5, x, yy);
	    }
    }

    // designing a spirale
    public void showVisibility(int x, int y, int v)
    {
	int i, xx, yy, step, doingStep;

	step=1;
	doingStep=0;
	directionVector[0] = 0;
	directionVector[1] = 1;
	xx=x;
	yy=y;

	for (i=1;i<=v;i++)
	    {
		doingStep++;
		xx+=directionVector[0];
		yy+=directionVector[1];

		xx=CoordinateManager.check(getSizeX(), xx);
		yy=CoordinateManager.check(getSizeY(), yy);
		if(getValueAtX$Y(xx,yy)==0)
		    putValue$atX$Y(2, xx, yy);

		if (doingStep == step || doingStep == 2*step) rotate();
		if (doingStep == 2*step) 
		    {
			doingStep=0;
			step++;
		    }

 	    }
    }


    public void rotate (){

	directionVector0[0]=directionVector[0];
	directionVector0[1]=directionVector[1];

	directionVector[0]=rotationMatrix[0][0]*directionVector0[0] + 
	                   rotationMatrix[0][1]*directionVector0[1];
	directionVector[1]=rotationMatrix[1][0]*directionVector0[0] + 
	                   rotationMatrix[1][1]*directionVector0[1];
	
    }

    /** the measure of the common positions in the visibility areas of 
     *  two units */
    public int interVisibility(Unit a, Unit b)
    {
	int interVisibility, i, j, xx, yy, step, doingStep;
	interVisibility=0;
	for (i=0;i<xSize;i++)
	    for (j=0;j<ySize;j++)
		{
		    aSpace [i][j]=0;
		    bSpace [i][j]=0;
		}

	// Unit a
	step=1;
	doingStep=0;
	directionVector[0] = 0;
	directionVector[1] = 1;
	xx=a.getXPos();
	yy=a.getYPos();

	for (i=1;i<=a.getVisibility();i++)
	    {
		doingStep++;
		xx+=directionVector[0];
		yy+=directionVector[1];

		xx=CoordinateManager.check(getSizeX(), xx);
		yy=CoordinateManager.check(getSizeY(), yy);
		aSpace[xx][yy]=1;

		if (doingStep == step || doingStep == 2*step) rotate();
		if (doingStep == 2*step) 
		    {
			doingStep=0;
			step++;
		    }
 	    } // end Unit a

	// Unit b
	step=1;
	doingStep=0;
	directionVector[0] = 0;
	directionVector[1] = 1;
	xx=b.getXPos();
	yy=b.getYPos();

	for (i=1;i<=b.getVisibility();i++)
	    {
		doingStep++;
		xx+=directionVector[0];
		yy+=directionVector[1];

		xx=CoordinateManager.check(getSizeX(), xx);
		yy=CoordinateManager.check(getSizeY(), yy);
		bSpace[xx][yy]=1;

		if (doingStep == step || doingStep == 2*step) rotate();
		if (doingStep == 2*step) 
		    {
			doingStep=0;
			step++;
		    }
 	    } // end Unit a

	for (i=0;i<xSize;i++)
	    for (j=0;j<ySize;j++)
		{
		    if(aSpace[i][j]==1 && bSpace[i][j]==1)interVisibility++;
		}

	return interVisibility;
    }

    /** return xSize */
    public int getXSize()
    {
	return xSize;
    }

    /** return ySize */
    public int getYSize()
    {
	return ySize;
    }

}
